"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var couchbase_plugin_common_1 = require("./couchbase-plugin.common");
var types = require("tns-core-modules/utils/types");
var fs = require("tns-core-modules/file-system");
var couchbase_plugin_common_2 = require("./couchbase-plugin.common");
exports.QueryMeta = couchbase_plugin_common_2.QueryMeta;
exports.QueryArrayOperator = couchbase_plugin_common_2.QueryArrayOperator;
exports.QueryLogicalOperator = couchbase_plugin_common_2.QueryLogicalOperator;
var Couchbase = (function (_super) {
    __extends(Couchbase, _super);
    function Couchbase(databaseName) {
        var _this = _super.call(this, databaseName) || this;
        _this._listenerMap = {};
        _this.ios = CBLDatabase.alloc().initWithNameError(databaseName);
        return _this;
    }
    Couchbase.prototype.inBatch = function (batch) {
        var errorRef = new interop.Reference();
        this.ios.inBatchUsingBlock(errorRef, batch);
    };
    Couchbase.prototype.createDocument = function (data, documentId) {
        var doc;
        if (documentId) {
            doc = CBLMutableDocument.alloc().initWithID(documentId);
        }
        else {
            doc = CBLMutableDocument.alloc().init();
        }
        var keys = Object.keys(data);
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            var item = data[key];
            this.serialize(item, doc, key);
        }
        this.ios.saveDocumentError(doc);
        return doc.id;
    };
    Couchbase.prototype.setBlob = function (id, name, blob, mimeType) {
        if (mimeType === void 0) { mimeType = 'application/octet-stream'; }
        try {
            var document_1 = this.ios.documentWithID(id).toMutable();
            if (typeof blob === 'string') {
                if (blob.startsWith("file")) {
                    var nativeBlob = CBLBlob.alloc().initWithContentTypeFileURLError(mimeType, NSURL.URLWithString(blob));
                    document_1.setBlobForKey(nativeBlob, name);
                }
                else if (blob.startsWith("/")) {
                    var nativeBlob = CBLBlob.alloc().initWithContentTypeFileURLError(mimeType, NSURL.fileURLWithPath(blob));
                    document_1.setBlobForKey(nativeBlob, name);
                }
                else if (blob.startsWith("~")) {
                    var path = fs.path.join(fs.knownFolders.currentApp().path, blob.replace('~', ''));
                    var nativeBlob = CBLBlob.alloc().initWithContentTypeFileURLError(mimeType, NSURL.URLWithString(path));
                    document_1.setBlobForKey(nativeBlob, name);
                }
                else if (blob.startsWith("res")) {
                    var bundle = NSBundle.mainBundle;
                    var path = bundle.pathForResourceOfType(blob.replace('res://', ''), this.getExtension(mimeType));
                    var nativeBlob = CBLBlob.alloc().initWithContentTypeFileURLError(mimeType, NSURL.URLWithString(path));
                    document_1.setBlobForKey(nativeBlob, name);
                }
                else {
                }
                this.ios.saveDocumentError(document_1);
            }
            else {
            }
        }
        catch (e) {
            console.debug(e);
        }
    };
    Couchbase.prototype.getBlob = function (id, name) {
        var document = this.ios.documentWithID(id);
        if (!document)
            return null;
        var blob = document.blobForKey(name);
        if (!blob)
            return null;
        return new Blob(blob);
    };
    Couchbase.prototype.fromISO8601UTC = function (date) {
        var dateFormatter = NSDateFormatter.new();
        dateFormatter.dateFormat = 'yyyy-MM-dd\'T\'HH:mm:ss.SSSZ';
        return dateFormatter.dateFromString(date);
    };
    Couchbase.prototype.serializeObject = function (item, object, key) {
        var _this = this;
        if (item === null) {
            object.setValueForKey(null, key);
            return;
        }
        switch (typeof item) {
            case 'object':
                if (item instanceof Date) {
                    object.setDateForKey(this.fromISO8601UTC(item.toISOString()), key);
                    return;
                }
                if (Array.isArray(item)) {
                    var array_1 = CBLMutableArray.new();
                    item.forEach(function (data) {
                        _this.serializeArray(data, array_1);
                    });
                    object.setArrayForKey(array_1, key);
                    return;
                }
                var nativeObject_1 = CBLMutableDictionary.new();
                Object.keys(item).forEach(function (itemKey) {
                    var obj = item[itemKey];
                    _this.serializeObject(obj, nativeObject_1, itemKey);
                });
                object.setDictionaryForKey(nativeObject_1, key);
                break;
            case 'number':
                if (this.numberIs64Bit(item)) {
                    if (this.numberHasDecimals(item)) {
                        object.setDoubleForKey(item, key);
                    }
                    else {
                        object.setLongLongForKey(item, key);
                    }
                }
                else {
                    if (this.numberHasDecimals(item)) {
                        object.setFloatForKey(item, key);
                    }
                    else {
                        object.setIntegerForKey(item, key);
                    }
                }
                break;
            case 'boolean':
                object.setBooleanForKey(item, key);
                break;
            default:
                object.setValueForKey(item, key);
        }
    };
    Couchbase.prototype.serializeArray = function (item, array) {
        var _this = this;
        if (item === null) {
            array.addValue(null);
            return;
        }
        switch (typeof item) {
            case 'object':
                if (item instanceof Date) {
                    array.addDate(this.fromISO8601UTC(item.toISOString()));
                    return;
                }
                if (Array.isArray(item)) {
                    var nativeArray_1 = CBLMutableArray.new();
                    item.forEach(function (data) {
                        _this.serializeArray(data, nativeArray_1);
                    });
                    array.addArray(nativeArray_1);
                    return;
                }
                var object_1 = CBLMutableDictionary.new();
                Object.keys(item).forEach(function (itemKey) {
                    var obj = item[itemKey];
                    _this.serializeObject(obj, object_1, itemKey);
                });
                array.addDictionary(object_1);
                break;
            case 'number':
                if (this.numberIs64Bit(item)) {
                    if (this.numberHasDecimals(item)) {
                        array.addDouble(item);
                    }
                    else {
                        array.addLongLong(item);
                    }
                }
                else {
                    if (this.numberHasDecimals(item)) {
                        array.addFloat(item);
                    }
                    else {
                        array.addInteger(item);
                    }
                }
                break;
            case 'boolean':
                array.addBoolean(item);
                break;
            default:
                array.addValue(item);
        }
    };
    Couchbase.prototype.serialize = function (item, doc, key) {
        var _this = this;
        if (item === null) {
            doc.setValueForKey(null, key);
            return;
        }
        switch (typeof item) {
            case 'object':
                if (item instanceof Date) {
                    doc.setDateForKey(this.fromISO8601UTC(item.toISOString()), key);
                    return;
                }
                if (Array.isArray(item)) {
                    var array_2 = CBLMutableArray.new();
                    item.forEach(function (data) {
                        _this.serializeArray(data, array_2);
                    });
                    doc.setArrayForKey(array_2, key);
                    return;
                }
                var object_2 = CBLMutableDictionary.new();
                Object.keys(item).forEach(function (itemKey) {
                    var obj = item[itemKey];
                    _this.serializeObject(obj, object_2, itemKey);
                });
                doc.setDictionaryForKey(object_2, key);
                break;
            case 'number':
                if (this.numberIs64Bit(item)) {
                    if (this.numberHasDecimals(item)) {
                        doc.setDoubleForKey(item, key);
                    }
                    else {
                        doc.setLongLongForKey(item, key);
                    }
                }
                else {
                    if (this.numberHasDecimals(item)) {
                        doc.setFloatForKey(item, key);
                    }
                    else {
                        doc.setIntegerForKey(item, key);
                    }
                }
                break;
            case 'boolean':
                doc.setBooleanForKey(item, key);
                break;
            default:
                doc.setValueForKey(item, key);
        }
    };
    Couchbase.prototype.getDocument = function (documentId) {
        var doc = this.ios.documentWithID(documentId);
        if (doc) {
            var obj = {};
            var keys = doc.keys;
            var size = keys.count;
            obj['id'] = doc.id;
            for (var i = 0; i < size; i++) {
                var key = keys.objectAtIndex(i);
                var value = doc.valueForKey(key);
                var newValue = {};
                newValue[key] = this.deserialize(value);
                obj = Object.assign(obj, newValue);
            }
            return obj;
        }
        return null;
    };
    Couchbase.prototype.numberHasDecimals = function (item) {
        return !(item % 1 === 0);
    };
    Couchbase.prototype.numberIs64Bit = function (item) {
        return item < -Math.pow(2, 31) + 1 || item > Math.pow(2, 31) - 1;
    };
    Couchbase.prototype.deserialize = function (data) {
        if (typeof data === 'string' || typeof data === 'number' || typeof data === 'boolean' || typeof data !== 'object')
            return data;
        if (types.isNullOrUndefined(data)) {
            return data;
        }
        if (data instanceof NSNull) {
            return null;
        }
        if (data instanceof CBLDictionary) {
            var keys = data.keys;
            var length_1 = keys.count;
            var object = {};
            for (var i = 0; i < length_1; i++) {
                var key = keys.objectAtIndex(i);
                var nativeItem = data.valueForKey(key);
                object[key] = this.deserialize(nativeItem);
            }
            return object;
        }
        if (data instanceof CBLArray) {
            var array = [];
            var size = data.count;
            for (var i = 0; i < size; i++) {
                var nativeItem = data.valueAtIndex(i);
                var item = this.deserialize(nativeItem);
                array.push(item);
            }
            return array;
        }
        return data;
    };
    Couchbase.prototype.updateDocument = function (documentId, data) {
        var original = this.ios.documentWithID(documentId);
        var newDoc = original.toMutable();
        var keys = Object.keys(data);
        for (var _i = 0, keys_2 = keys; _i < keys_2.length; _i++) {
            var key = keys_2[_i];
            var item = data[key];
            this.serialize(item, newDoc, key);
        }
        this.ios.saveDocumentError(newDoc);
    };
    Couchbase.prototype.deleteDocument = function (documentId, concurrencyMode) {
        if (concurrencyMode === void 0) { concurrencyMode = 1; }
        var doc = this.ios.documentWithID(documentId);
        return this.ios.deleteDocumentConcurrencyControlError(doc, concurrencyMode === 1 ? 1 :
            0);
    };
    Couchbase.prototype.destroyDatabase = function () {
        return this.ios.delete();
    };
    Couchbase.prototype.createReplication = function (remoteUrl, direction) {
        var url = NSURL.alloc().initWithString(remoteUrl);
        var targetEndpoint = CBLURLEndpoint.alloc().initWithURL(url);
        var replConfig = CBLReplicatorConfiguration.alloc().initWithDatabaseTarget(this.ios, targetEndpoint);
        if (direction === 'pull') {
            replConfig.replicatorType = CBLReplicatorType.kCBLReplicatorTypePull;
        }
        else if (direction === 'push') {
            replConfig.replicatorType = CBLReplicatorType.kCBLReplicatorTypePush;
        }
        else {
            replConfig.replicatorType = CBLReplicatorType.kCBLReplicatorTypePushAndPull;
        }
        var replicator = CBLReplicator.alloc().initWithConfig(replConfig);
        return new Replicator(replicator);
    };
    Couchbase.prototype.createPushReplication = function (remoteUrl) {
        return this.createReplication(remoteUrl, 'push');
    };
    Couchbase.prototype.createPullReplication = function (remoteUrl) {
        return this.createReplication(remoteUrl, 'pull');
    };
    Couchbase.prototype.addDatabaseChangeListener = function (callback) {
        var token = this.ios.addChangeListener(function (change) {
            if (callback && typeof callback === 'function') {
                var ids = [];
                var documentIds = change.documentIDs;
                var size = documentIds.count;
                for (var i = 0; i < size; i++) {
                    var item = documentIds[i];
                    ids.push(item);
                }
                callback(ids);
            }
        });
        if (!types.isNullOrUndefined(token)) {
            this._listenerMap[callback] = token;
        }
    };
    Couchbase.prototype.removeDatabaseChangeListener = function (callback) {
        var token = this._listenerMap[callback];
        if (!types.isNullOrUndefined(token)) {
            this.ios.removeChangeListenerWithToken(token);
            delete this._listenerMap[callback];
        }
    };
    Couchbase.prototype.serializeExpression = function (item) {
        if (item === null) {
            return null;
        }
        switch (typeof item) {
            case 'string':
                return CBLQueryExpression.string(item);
            case 'object':
                if (item instanceof Date) {
                    return CBLQueryExpression.date(this.fromISO8601UTC(item.toISOString()));
                }
                return CBLQueryExpression.value(item);
            case 'number':
                if (this.numberIs64Bit(item)) {
                    if (this.numberHasDecimals(item)) {
                        return CBLQueryExpression.double(item);
                    }
                    else {
                        return CBLQueryExpression.longLong(item);
                    }
                }
                else {
                    if (this.numberHasDecimals(item)) {
                        return CBLQueryExpression.float(item);
                    }
                    else {
                        return CBLQueryExpression.integer(item);
                    }
                }
            case 'boolean':
                return CBLQueryExpression.boolean(item);
            default:
                return CBLQueryExpression.value(item);
        }
    };
    Couchbase.prototype.setComparision = function (item) {
        var nativeQuery;
        switch (item.comparison) {
            case 'equalTo':
                nativeQuery = CBLQueryExpression.property(item.property).equalTo(this.serializeExpression(item.value));
                break;
            case 'add':
                nativeQuery = CBLQueryExpression.property(item.property).add(this.serializeExpression(item.value));
                break;
            case 'between':
                if (Array.isArray(item.value) && item.value.length === 2) {
                    nativeQuery = CBLQueryExpression.property(item.property).between(this.serializeExpression(item.value[0])).andExpression(this.serializeExpression(item.value[1]));
                }
                break;
            case 'collate':
                nativeQuery = CBLQueryExpression.property(item.property).collate(this.serializeExpression(item.value));
                break;
            case 'divide':
                nativeQuery = CBLQueryExpression.property(item.property).divide(this.serializeExpression(item.value));
                break;
            case 'greaterThan':
                nativeQuery = CBLQueryExpression.property(item.property).greaterThan(this.serializeExpression(item.value));
                break;
            case 'greaterThanOrEqualTo':
                nativeQuery = CBLQueryExpression.property(item.property).greaterThanOrEqualTo(this.serializeExpression(item.value));
                break;
            case 'in':
                var inArray = [];
                if (Array.isArray(item.value)) {
                    for (var _i = 0, _a = item.value; _i < _a.length; _i++) {
                        var exp_1 = _a[_i];
                        inArray.push(this.serializeExpression(exp_1));
                    }
                }
                else {
                    inArray.push(this.serializeExpression(item.value));
                }
                nativeQuery = CBLQueryExpression.property(item.property).in(inArray);
                break;
            case 'is':
                nativeQuery = CBLQueryExpression.property(item.property).is(this.serializeExpression(item.value));
                break;
            case 'isNot':
                nativeQuery = CBLQueryExpression.property(item.property).isNot(this.serializeExpression(item.value));
                break;
            case 'isNullOrMissing':
                nativeQuery = CBLQueryExpression.property(item.property).isNullOrMissing();
                break;
            case 'lessThan':
                nativeQuery = CBLQueryExpression.property(item.property).lessThan(this.serializeExpression(item.value));
                break;
            case 'lessThanOrEqualTo':
                nativeQuery = CBLQueryExpression.property(item.property).lessThanOrEqualTo(this.serializeExpression(item.value));
                break;
            case 'like':
                nativeQuery = CBLQueryFunction.lower(CBLQueryExpression.property(item.property)).like(this.serializeExpression(item.value));
                break;
            case 'modulo':
                nativeQuery = CBLQueryExpression.property(item.property).modulo(this.serializeExpression(item.value));
                break;
            case 'multiply':
                nativeQuery = CBLQueryExpression.property(item.property).multiply(this.serializeExpression(item.value));
                break;
            case 'notEqualTo':
                nativeQuery = CBLQueryExpression.property(item.property).notEqualTo(this.serializeExpression(item.value));
                break;
            case 'notNullOrMissing':
                nativeQuery = CBLQueryExpression.property(item.property).notNullOrMissing();
                break;
            case 'regex':
                nativeQuery = CBLQueryFunction.lower(item.property).regex(this.serializeExpression(item.value));
                break;
        }
        return nativeQuery;
    };
    Couchbase.prototype.query = function (query) {
        if (query === void 0) { query = { select: [couchbase_plugin_common_1.QueryMeta.ALL, couchbase_plugin_common_1.QueryMeta.ID] }; }
        var items = [];
        var select = [];
        var from;
        var join = null;
        var where;
        var groupBy = null;
        var orderBy = null;
        var limit = null;
        var having = null;
        var isAll = false;
        if (!query.select || query.select.length === 0) {
            isAll = true;
            select.push(CBLQuerySelectResult.all());
            select.push(CBLQuerySelectResult.expression(CBLQueryMeta.id()));
        }
        else {
            query.select.forEach(function (item) {
                if (item === couchbase_plugin_common_1.QueryMeta.ID) {
                    select.push(CBLQuerySelectResult.expression(CBLQueryMeta.id()));
                }
                else if (item === couchbase_plugin_common_1.QueryMeta.ALL) {
                    isAll = true;
                    select.push(CBLQuerySelectResult.all());
                }
                else {
                    select.push(CBLQueryExpression.property(item));
                }
            });
        }
        if (query.from) {
            var db = new Couchbase(query.from);
            from = CBLQueryDataSource.database(db.ios);
        }
        else {
            from = CBLQueryDataSource.database(this.ios);
        }
        var nativeQuery = null;
        if (query.where) {
            for (var _i = 0, _a = query.where; _i < _a.length; _i++) {
                var item = _a[_i];
                if (item.logical === couchbase_plugin_common_1.QueryLogicalOperator.AND) {
                    if (!nativeQuery)
                        break;
                    nativeQuery = nativeQuery.andExpression(this.setComparision(item));
                }
                else if (item.logical === couchbase_plugin_common_1.QueryLogicalOperator.OR) {
                    if (!nativeQuery)
                        break;
                    nativeQuery = nativeQuery.orExpression(this.setComparision(item));
                }
                else {
                    nativeQuery = this.setComparision(item);
                }
            }
        }
        if (query.groupBy) {
            if (query.groupBy.length > 0) {
                groupBy = [];
            }
            for (var _b = 0, _c = query.groupBy; _b < _c.length; _b++) {
                var prop = _c[_b];
                groupBy.push(CBLQueryExpression.property(prop));
            }
        }
        if (query.order) {
            if (query.order.length > 0) {
                orderBy = [];
            }
            for (var _d = 0, _e = query.order; _d < _e.length; _d++) {
                var item = _e[_d];
                switch (item.direction) {
                    case 'desc':
                        orderBy.push(CBLQueryOrdering.property(item.property).descending());
                        break;
                    default:
                        orderBy.push(CBLQueryOrdering.property(item.property).ascending());
                        break;
                }
            }
        }
        if (query.limit && typeof query.limit === 'number') {
            if (query.offset && typeof query.offset === 'number') {
                limit = CBLQueryLimit.limitOffset(CBLQueryExpression.integer(query.limit), CBLQueryExpression.integer(query.offset));
            }
            else {
                limit = CBLQueryLimit.limit(CBLQueryExpression.integer(query.limit));
            }
        }
        var queryBuilder = CBLQueryBuilder.selectFromJoinWhereGroupByHavingOrderByLimit(select, from, join, nativeQuery, groupBy, having, orderBy, limit);
        var result = queryBuilder.execute().allResults();
        var size = result.count;
        for (var i = 0; i < size; i++) {
            var item = result.objectAtIndex(i);
            var keys = item.keys;
            var keysSize = keys.count;
            var obj = {};
            for (var keyId = 0; keyId < keysSize; keyId++) {
                var key = keys.objectAtIndex(keyId);
                var nativeItem = item.valueForKey(key);
                if (isAll && types.getClass(nativeItem) === 'CBLDictionary') {
                    var cblKeys = nativeItem.keys;
                    var cblKeysSize = cblKeys.count;
                    for (var cblKeysId = 0; cblKeysId < cblKeysSize; cblKeysId++) {
                        var cblKey = cblKeys.objectAtIndex(cblKeysId);
                        obj[cblKey] = this.deserialize(nativeItem.valueForKey(cblKey));
                    }
                }
                else {
                    obj[key] = this.deserialize(nativeItem);
                }
            }
            items.push(obj);
        }
        return items;
    };
    Couchbase.prototype.getExtension = function (mimeType) {
        if (!mimeType)
            return 'bin';
        for (var _i = 0, _a = Object.keys(mimeTypes); _i < _a.length; _i++) {
            var key = _a[_i];
            var mime = mimeType[key];
            if (typeof mimeType === 'string' && mimeType.indexOf(mime) > -1) {
                return key;
            }
        }
        return 'bin';
    };
    return Couchbase;
}(couchbase_plugin_common_1.Common));
exports.Couchbase = Couchbase;
var Replicator = (function (_super) {
    __extends(Replicator, _super);
    function Replicator(replicator) {
        return _super.call(this, replicator) || this;
    }
    Replicator.prototype.start = function () {
        this.replicator.start();
    };
    Replicator.prototype.stop = function () {
        this.replicator.stop();
    };
    Replicator.prototype.isRunning = function () {
        if (this.replicator && this.replicator.status && this.replicator.status.activity) {
            return (this.replicator.status.activity ===
                CBLReplicatorActivityLevel.kCBLReplicatorBusy);
        }
        return false;
    };
    Replicator.prototype.setContinuous = function (isContinuous) {
        var newConfig = CBLReplicatorConfiguration.alloc().initWithConfig(this.replicator.config);
        newConfig.continuous = isContinuous;
        this.replicator = CBLReplicator.alloc().initWithConfig(newConfig);
    };
    Replicator.prototype.setUserNameAndPassword = function (username, password) {
        var newConfig = CBLReplicatorConfiguration.alloc().initWithConfig(this.replicator.config);
        newConfig.authenticator = CBLBasicAuthenticator.alloc().initWithUsernamePassword(username, password);
        this.replicator = CBLReplicator.alloc().initWithConfig(newConfig);
    };
    Replicator.prototype.setChannels = function (channels) {
        var newConfig = CBLReplicatorConfiguration.alloc().initWithConfig(this.replicator.config);
        newConfig.channels = NSArray.arrayWithArray(channels);
        this.replicator = CBLReplicator.alloc().initWithConfig(newConfig);
    };
    Replicator.prototype.setSessionIdAndCookieName = function (sessionId, cookieName) {
        var newConfig = CBLReplicatorConfiguration.alloc().initWithConfig(this.replicator.config);
        newConfig.authenticator = CBLSessionAuthenticator.alloc().initWithSessionIDCookieName(sessionId, cookieName);
        this.replicator = CBLReplicator.alloc().initWithConfig(newConfig);
    };
    Replicator.prototype.setSessionId = function (sessionId) {
        var newConfig = CBLReplicatorConfiguration.alloc().initWithConfig(this.replicator.config);
        newConfig.authenticator = CBLSessionAuthenticator.alloc().initWithSessionID(sessionId);
        this.replicator = CBLReplicator.alloc().initWithConfig(newConfig);
    };
    return Replicator;
}(couchbase_plugin_common_1.ReplicatorBase));
exports.Replicator = Replicator;
var Blob = (function (_super) {
    __extends(Blob, _super);
    function Blob(blob) {
        return _super.call(this, blob) || this;
    }
    Object.defineProperty(Blob.prototype, "ios", {
        get: function () {
            return this.blob;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Blob.prototype, "content", {
        get: function () {
            if (!this.ios)
                return null;
            return this.ios.content;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Blob.prototype, "contentStream", {
        get: function () {
            if (!this.ios)
                return null;
            return this.ios.contentStream;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Blob.prototype, "contentType", {
        get: function () {
            if (!this.ios)
                return null;
            return this.ios.contentType;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Blob.prototype, "length", {
        get: function () {
            if (!this.ios)
                return 0;
            return this.ios.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Blob.prototype, "digest", {
        get: function () {
            if (!this.ios)
                return null;
            return this.ios.digest;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Blob.prototype, "properties", {
        get: function () {
            var map = new Map();
            if (!this.ios)
                return map;
            var nativeMap = this.ios.properties;
            var mapKeysArray = nativeMap.allKeys;
            var length = mapKeysArray.count;
            for (var i = 0; i < length; i++) {
                var key = mapKeysArray[i];
                var value = nativeMap.valueForKey(key);
                map.set(key, value);
            }
            return map;
        },
        enumerable: true,
        configurable: true
    });
    return Blob;
}(couchbase_plugin_common_1.BlobBase));
exports.Blob = Blob;
var DEFAULT_MIME_TYPE = 'application/octet-stream';
var mimeTypes = {
    'md': 'text/markdown',
    'html': 'text/html',
    'htm': 'text/html',
    'shtml': 'text/html',
    'css': 'text/css',
    'xml': 'text/xml',
    'gif': 'image/gif',
    'jpeg': 'image/jpeg',
    'jpg': 'image/jpeg',
    'js': 'application/javascript',
    'atom': 'application/atom+xml',
    'rss': 'application/rss+xml',
    'mml': 'text/mathml',
    'txt': 'text/plain',
    'jad': 'text/vnd.sun.j2me.app-descriptor',
    'wml': 'text/vnd.wap.wml',
    'htc': 'text/x-component',
    'png': 'image/png',
    'tif': 'image/tiff',
    'tiff': 'image/tiff',
    'wbmp': 'image/vnd.wap.wbmp',
    'ico': 'image/x-icon',
    'jng': 'image/x-jng',
    'bmp': 'image/x-ms-bmp',
    'svg': 'image/svg+xml',
    'svgz': 'image/svg+xml',
    'webp': 'image/webp',
    'woff': 'application/font-woff',
    'jar': 'application/java-archive',
    'war': 'application/java-archive',
    'ear': 'application/java-archive',
    'json': 'application/json',
    'hqx': 'application/mac-binhex40',
    'doc': 'application/msword',
    'pdf': 'application/pdf',
    'ps': 'application/postscript',
    'eps': 'application/postscript',
    'ai': 'application/postscript',
    'rtf': 'application/rtf',
    'm3u8': 'application/vnd.apple.mpegurl',
    'xls': 'application/vnd.ms-excel',
    'eot': 'application/vnd.ms-fontobject',
    'ppt': 'application/vnd.ms-powerpoint',
    'wmlc': 'application/vnd.wap.wmlc',
    'kml': 'application/vnd.google-earth.kml+xml',
    'kmz': 'application/vnd.google-earth.kmz',
    '7z': 'application/x-7z-compressed',
    'cco': 'application/x-cocoa',
    'jardiff': 'application/x-java-archive-diff',
    'jnlp': 'application/x-java-jnlp-file',
    'run': 'application/x-makeself',
    'pl': 'application/x-perl',
    'pm': 'application/x-perl',
    'prc': 'application/x-pilot',
    'pdb': 'application/x-pilot',
    'rar': 'application/x-rar-compressed',
    'rpm': 'application/x-redhat-package-manager',
    'sea': 'application/x-sea',
    'swf': 'application/x-shockwave-flash',
    'sit': 'application/x-stuffit',
    'tcl': 'application/x-tcl',
    'tk': 'application/x-tcl',
    'der': 'application/x-x509-ca-cert',
    'pem': 'application/x-x509-ca-cert',
    'crt': 'application/x-x509-ca-cert',
    'xpi': 'application/x-xpinstall',
    'xhtml': 'application/xhtml+xml',
    'xspf': 'application/xspf+xml',
    'zip': 'application/zip',
    'bin': 'application/octet-stream',
    'exe': 'application/octet-stream',
    'dll': 'application/octet-stream',
    'deb': 'application/octet-stream',
    'dmg': 'application/octet-stream',
    'iso': 'application/octet-stream',
    'img': 'application/octet-stream',
    'msi': 'application/octet-stream',
    'msp': 'application/octet-stream',
    'msm': 'application/octet-stream',
    'docx': 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
    'xlsx': 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
    'pptx': 'application/vnd.openxmlformats-officedocument.presentationml.presentation',
    'mid': 'audio/midi',
    'midi': 'audio/midi',
    'kar': 'audio/midi',
    'mp3': 'audio/mpeg',
    'ogg': 'audio/ogg',
    'm4a': 'audio/x-m4a',
    'ra': 'audio/x-realaudio',
    '3gpp': 'video/3gpp',
    '3gp': 'video/3gpp',
    'ts': 'video/mp2t',
    'mp4': 'video/mp4',
    'mpeg': 'video/mpeg',
    'mpg': 'video/mpeg',
    'mov': 'video/quicktime',
    'webm': 'video/webm',
    'flv': 'video/x-flv',
    'm4v': 'video/x-m4v',
    'mng': 'video/x-mng',
    'asx': 'video/x-ms-asf',
    'asf': 'video/x-ms-asf',
    'wmv': 'video/x-ms-wmv',
    'avi': 'video/x-msvideo'
};
//# sourceMappingURL=couchbase-plugin.ios.js.map